// PreparedCommands.h: interface for the PreparedCommands class.
//
//////////////////////////////////////////////////////////////////////
#ifndef PLAYER_WORLD_WORLD_H
#define PLAYER_WORLD_WORLD_H

#include "Common/World/Point.h"

// SS: behavior of player dependent of SPusti message
typedef enum
{
	DEFAULT,		// default behavior
	FORCERUNTOBALL,	// after sending SPusti message - force run to ball!
	NOTRUNTOBALL	// after receiving SPusti message - not run to ball!
} SPustiBehavior;

class PreparedCommands  
{
public:
	void ResetAll();
	void ResetFindPassableFriend(){haveFindpassableFriend =false;};
	bool HaveFindPassableFriend(){return haveFindpassableFriend;};
	void FindPassableFriend(){haveFindpassableFriend =true;};
	void ResetWaitTime();
	bool HaveWaitTo(int cycle);
	void SetWaitTime(int time);
	bool HaveMoveTo(Point &pos);
	void ResetMoveTo();
	void SetMoveTo(Point pos);
	bool HaveTurnTo(Point &pos);
	void ResetTurnTo();
	void SetTurnTo(Point pos);
	bool HaveKick(Point &pos);
	void resetKickTo();
	void SetKickTo(Point &p);
	bool ReachTarget(Point pos);
	void Debug();
	void ResetTarget();
	void SetTarget(Point pos, float precision=1.0);

	PreparedCommands& operator =(const PreparedCommands &other);
	PreparedCommands(const PreparedCommands& other);
	PreparedCommands();
	virtual ~PreparedCommands();
	
	bool  activeTarget;		///< indikuje ci je target aktivny
	int nSteps;				///< pocet krokov, po ktory je aktivny target
    Point targetPos;		///< ciel kam sa ma dostat
    float targetPrecision;	///> presnost dosiahnutia ciela
	bool bKickTo;			///> mam v plane kopnut do lopty
	Point kickPosTo;		///> kopnut loptu na poziciu
	
	bool bTurnTo;			///> mam v planeo otocit sa
	Point turnToPos;		///> pozicia na ktoru mam byt otoceny

	bool bMoveTo;			/// mam niekde ist
	Point moveToPos;		/// kde mam ist

	int waitTime;			/// kolko simulacnych krokov nerobit nic
	
	/// indikuje ze ma najst spoluhraca ktoremu prihra
	bool haveFindpassableFriend;
	
	/* SS old
	bool notRunToBall;		/// do not run to ball (after receiving SPusti)
	bool forceRunToBall;	/// run to ball (after sending SPusti)
	*/

	// -----------------------------------
	// SS: SPusti message handling members
	// -----------------------------------
	
	bool ShouldRunToBall();		// checks if player should run to ball
	bool ForceRunToBall();		// checks if player sended SPusti message before
	bool DefaultRunToBall();	// check if default behavior is set
	float GetSPustiDistToBall(); // gets distance to ball when player sended last SPusti
	Vector GetSpustiBallVect();	// gets vector of ball when player sended last SPusti
	
	void SetForceRunToBall(float meBallDist, Vector* ballVector );	// sets sPustiBehaviorStatus to FORCERUNTOBALL
	void SetDefaultRunToBall();	// sets sPustiBehaviorStatus to DEFAULT
	void SetNotRunToBall(Vector* ballVector);		// sets sPustiBehaviorStatus to NOTRUNTOBALL

private:

	// -----------------------------------
	// SS: SPusti message handling members
	// -----------------------------------

	SPustiBehavior	sPustiBehaviorStatus;		// behavior of player dependent of SPusti message (see SPustiBehavior enum)
	float			sPustiBallDistance;			// distance to ball when sending SPusti message
	Vector			sPustiBallVector;			// vector of ball when sending or receiving SPusti message
	//int				ticksToCancelNotRun;		// ticks to cancel NOTRUNTOBALL status	
};

#endif // PLAYER_WORLD_WORLD_H
